<?php
/***
 * Plugin Name: Debug Log List
 * Description: Extracts errors from your debug.log and groups them into individual items.
 * Version: 0.3.3.3
 * Author: Jay
 * Author URI: https://profiles.wordpress.org/jayhybrid
 * Contributors: jayhybrid
 * Text Domain: hybridsupply-log
 */
defined('HYBRIDSUPPLY_LOG_FILE') or define('HYBRIDSUPPLY_LOG_FILE', __FILE__);

function hybridsupply_log_options_html() {
    if ( ! current_user_can( 'manage_options' ) ) {
      return;
    }

    /*
     * Read log file
     */
    $log = array();
    $log_hashes = array();

    $logfile = WP_DEBUG_LOG === TRUE ? WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'debug.log' : WP_DEBUG_LOG;
    $logfile_url = WP_DEBUG_LOG === TRUE ? content_url('debug.log') : content_url(WP_DEBUG_LOG);

    if ( file_exists($logfile) ) {
      /* Read file into an array */
      $logfile_contents = file( $logfile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES );

      /* Extract information per line */
      for ($i=0; $i < count($logfile_contents); $i++) {
        $n = count($log);
        $log[$n] = new stdClass();

        /* Original */
        $log[$n]->original = $logfile_contents[$i];

        /* Timestamp */
        if ( preg_match('/\d{2}-[a-zA-Z]{3}-\d{4} \d{2}:\d{2}:\d{2}/', $logfile_contents[$i], $matches) ) {
          $log[$n]->timestamp = $matches[0];
        }
        else {
          $log[$n]->timestamp = '';
        }

        /* Type */
        if ( preg_match('/\]\s*([\w \)\(]*)/', $logfile_contents[$i], $matches) ) {
          $log[$n]->type = $matches[1];
        }
        else {
          $log[$n]->type = '';
        }

        /* Message */
        if ( preg_match('/\]\s*[\w \(\)]*[:\.]? *(.+)/', $logfile_contents[$i], $matches) ) {
          $log[$n]->message = substr($matches[1], 0, 4) === '<!--' ? esc_html( $matches[1] ) : $matches[1];
        }
        else {
          $log[$n]->message = '';
        }

        /* Hash */
        if ( preg_match('/\]\s*[\w ]*\w\W+(.+)/', $logfile_contents[$i], $matches) ) {
          $log[$n]->hash = hash( 'crc32' , $log[$n]->message );
        }
        else {
          $log[$n]->hash = '';
        }

        /* Plugin */
        if ( preg_match('/plugins\/[^\/.]+/', $logfile_contents[$i], $matches) ) {
          $log[$n]->plugin = $matches[0];
        }
        else {
          $log[$n]->plugin = '';
        }

        /* Folder */
        if ( preg_match('/ \/[\w\/\-_\.]+/', $logfile_contents[$i], $matches) ) {
          $log[$n]->folder = $matches[0];
        }
        else {
          $log[$n]->folder = '';
        }

        /* File */
        if ( preg_match('/ \/[\w\/\-_\.]+/', $logfile_contents[$i], $matches) ) {
          $log[$n]->file = $matches[0];
        }
        else {
          $log[$n]->file = '';
        }

        /* Line */
        if ( preg_match('/on line \d+/', $logfile_contents[$i], $matches) ) {
          $log[$n]->line = $matches[0];
        }
        else {
          $log[$n]->line = '';
        }

        /* Occurrences */
        $log[$n]->occurrences = 1;

        /* Options */
        $log[$n]->options = '<form method="post"><input type="hidden" name="action" value="delete"><input type="hidden" name="hash" value="' . $log[$n]->hash . '"><button type="submit" class="button button">Remove</button></form>';
      }

      /*
       * Perform actions
       */
      if ( isset( $_POST['action'] ) ) {
        switch ( $_POST['action'] ) {
          case 'delete':
            if ( isset( $_POST['hash'] ) ) {
              $log_file_new = array();
              $log_tmp = array();

              for ($i=0; $i < count($log); $i++) {
                if ( $log[$i]->hash != $_POST['hash'] ) {
                  $log_file_new[] = $log[$i]->original . PHP_EOL;
                  $log_tmp[] = $log[$i];
                }
              }

              $log = $log_tmp;

              file_put_contents( $logfile, $log_file_new );
            }
            break;

          case 'delete_log':
            hybridsupply_log_count(array(
              'count' => 0,
              'count_all' => 0,
            ));

            unlink($logfile);
            break;
        }
      }
    }

    ?>
    <div class="wrap">
      <header>
        <h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
      </header>

      <main>
      <?php
      if ( isset($logfile_contents) && $logfile_contents && file_exists($logfile) ) {
        if ( $log ) { ?>
          <table>
          	<thead>
          		<tr>
          			<th class="time">Time</th>
          			<th class="type">Type</th>
                <th class="message">Message</th>
                <th class="plugin">Plugin</th>
                <th class="folder">Folder</th>
                <th class="file">File</th>
                <th class="line">Line</th>
          			<th class="occurrences">Occurrences</th>
          			<th class="options">Options</th>
          		</tr>
          	</thead>

          	<tbody>
          		<?php
          		for ($i=0; $i < count($log); $i++) {
          			if ( ! in_array($log[$i]->hash, $log_hashes) ) {
                  $log_hashes[] = $log[$i]->hash;
          				$table_row_classes = array(
          					'hsl_type',
          					'hsl_' . preg_replace( '/ /', '_', strtolower($log[$i]->type) ),
          				);

    	      			echo '<tr class="' . implode(' ', $table_row_classes) . '">';

    	      			foreach ($log[$i] as $key => $value) {
    	      				switch ($key) {
                      case 'original':
                        break;

    	      					case 'timestamp':
                        echo '<td>' . date(get_option('date_format') . ' ' . get_option('time_format'), strtotime( $value) ) . '</td>';
                        break;

                      case 'message':
                        echo '<td class="message">' . $value . '</td>';
                        break;

                      case 'plugin':
                        $folder = substr($value, 8) ? substr($value, 8) : NULL;
                        $plugin;

                        if ( $folder ) {
                          $files = glob(WP_PLUGIN_DIR . '/' . $folder . '/*.php');
                          $td = '';

                          for ($i=0; $i < count( $files ); $i++) {
                            $plugin = get_plugin_data( $files[$i], FALSE, FALSE );

                            if ( $plugin['Name'] != '' ) {
                              $td = $plugin['Name'];
                              break;
                            }
                          }

                          echo '<td>' . $td . '</td>';
                        }
                        else {
                          echo '<td></td>';
                        }
                        break;

                      case 'folder':
                        $length = strlen(ABSPATH);
                        $base = trim( substr($value, 0, $length + 1) );

                        if ( ABSPATH == $base ) {
                          echo '<td>' . dirname( substr($value, $length) ) . '</td>';
                        }
                        else {
                          echo '<td>' . dirname($value) . '</td>';
                        }

                        break;

                      case 'file':
                        echo '<td>' . basename($value) . '</td>';
                        break;

                      case 'line':
                        if ( is_numeric($value) ) {
                          echo '<td>' . number_format_i18n( substr($value, 8) ) . '</td>';
                        }
                        else {
                          echo '<td></td>';
                        }
                        break;

    	      					case 'occurrences':
                        $log[$i]->occurrences = number_format_i18n( array_count_values( array_column($log, 'hash') )[$log[$i]->hash] );

                        echo '<td>' . $log[$i]->occurrences . '</td>';
                        break;

    	      					case 'fullmatch':
    	      					case 'hash':
    	      						break;

                      case 'options':
                        echo '<td class="options">' . $value . '</td>';
                        break;

    	      					default:
    	      						echo '<td>' . $value . '</td>';
    	      						break;
    	      				}
    	      			}

    	      			echo '</tr>';
    	      		}
          		}

          		?>
          	</tbody>
          </table>
      <?php
        }
        else {
          echo '<h2>Unable to extract log file contents</h2>';
          echo '<p>Sorry about that.</p>';
          echo '<textarea>';
          echo var_dump($logfile_contents);
          echo '</textarea>';
        }
      ?>
      </main>

      <footer>
      <?php
        $filesize = filesize($logfile);
        $units1024 = array(
          'kibi',
          'mebi',
          'gibi',
          'tebi',
          'pebi',
          'exbi',
        );
        $units1000 = array(
          'kilo',
          'mega',
          'giga',
          'terra',
          'peta',
          'exa',
        );
        $sizesunits1024 = array();
        $sizesunits1000 = array();
        $byte = 'byte';

        for ($i=0; $i < count($units1024); $i++) {
          $current_size1024 = round($filesize / pow(1024, $i + 1), 2);
          $current_size1000 = round($filesize / pow(1000, $i + 1), 2);

          $byte = $current_size1024 == 1 ? 'byte' : 'bytes';

          if ( $current_size1024 >= 1 ) {
            $sizesunits1024[] = $current_size1024 . ' ' . $units1024[$i] . $byte;
            $sizesunits1000[] = $current_size1000 . ' ' . $units1000[$i] . $byte;
          }
        }

        if ( ! $sizesunits1024 ) {
          $sizesunits1024 = array($filesize . ' ' . $byte);
          $sizesunits1000 = array($filesize . ' ' . $byte);
        }

        echo '<div class="row">';
        echo '<form method="post"><input type="hidden" name="action" value="delete_log"><button type="submit" class="button button-primary">Delete Log</button></form>';
        echo WP_DEBUG_LOG === TRUE ? '<a href="' . $logfile_url . '" class="button" target="_blank">View Raw Log</a>' : '';
        echo '<p class="row_end">Log file size: <span data-tooltip="' . min( $sizesunits1000 ) . '">' . min( $sizesunits1024 ) . '</span>.</p>';
        echo '</div>';
      }
      else {
        echo '<p>' . $logfile . ' seems to be empty.</p>';

        hybridsupply_log_count(array(
          'count' => 0,
          'count_all' => 0,
        ));
      }
      ?>
      </footer>
    </div>
    <?php
}

function hybridsupply_log_options_menu() {
  $hybridsupply_log = array();
	$hybridsupply_log_count = get_option( 'hybridsupply_log_count') ? sprintf('<span class="awaiting-mod">%d</span>', get_option('hybridsupply_log_count') ) : '';
	$hybridsupply_log_count_all = get_option( 'hybridsupply_log_count_all') ? sprintf('<span class="awaiting-mod">%d</span>', get_option('hybridsupply_log_count_all') ) : '';

  if ( $hybridsupply_log_count ) { $hybridsupply_log[] = $hybridsupply_log_count; }
  /* if ( $hybridsupply_log_count != $hybridsupply_log_count_all ) { $hybridsupply_log[] = $hybridsupply_log_count_all; } */

	$hybridsupply_log = $hybridsupply_log ? 'Log ' . implode(' / ', $hybridsupply_log) : 'Log';

  add_menu_page(
      'HybridSupply Log',
      $hybridsupply_log,
      'manage_options',
      'hybridsupply-log',
      'hybridsupply_log_options_html',
      'dashicons-media-text'
  );
}

add_action( 'admin_menu', 'hybridsupply_log_options_menu' );


/**
 * Hooks
 */
$hybridsupply_log_include_hook_folder = array(
	'includes',
	'hook',
);


/*
 * Hook: Enqueue scripts
 */
include_once implode(DIRECTORY_SEPARATOR, $hybridsupply_log_include_hook_folder) . DIRECTORY_SEPARATOR . 'hybridsupply_log_enqueue_scripts_and_styles.php';


/*
 * Hook: Cron
 */
include_once implode(DIRECTORY_SEPARATOR, $hybridsupply_log_include_hook_folder) . DIRECTORY_SEPARATOR . 'hybridsupply_log_cron.php';


/**
 * Functions
 */
$hybridsupply_log_include_function_folder = array(
  'includes',
  'function',
);


/*
 * Function: Update Log Count
 */
function hybridsupply_log_count( $args ) {
  $count_unique = isset( $args['count'] ) ? $args['count'] : NULL;
  $count_all = isset( $args['count_all'] ) ? $args['count_all'] : NULL;

  /* Unique Error Count */
  if ( $count_unique !== NULL ) {
    if ( get_option('hybridsupply_log_count') === FALSE ) {
      add_option( 'hybridsupply_log_count', $count_unique );
    }
    else {
      update_option( 'hybridsupply_log_count', $count_unique );
    }
  }

  /* Error Count (All) */
  if ( $count_all !== NULL ) {
    if ( get_option('hybridsupply_log_count_all') === FALSE ) {
      add_option( 'hybridsupply_log_count_all', $count_all );
    }
    else {
      update_option( 'hybridsupply_log_count_all', $count_all );
    }
  }
}

/*
 * Function: Extract Log
 */
function hybridsupply_log_count_determine() {
  $log = array();
  $log_hashes = array();

  $logfile = WP_DEBUG_LOG === TRUE ? WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'debug.log' : WP_DEBUG_LOG;

  if ( file_exists($logfile) ) {
    /* Read file into an array */
    $logfile_contents = file( $logfile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES );

    /* Extract information per line */
    for ($i=0; $i < count($logfile_contents); $i++) {
      $n = count($log);
      $log[$n] = new stdClass();

      /* Message */
      if ( preg_match('/\]\s*[\w ]*\w\W+(.+)/', $logfile_contents[$i], $matches) ) {
        $log[$n]->message = substr($matches[1], 0, 4) === '<!--' ? esc_html( $matches[1] ) : $matches[1];
      }
      else {
        $log[$n]->message = '';
      }

      /* Hash */
      if ( preg_match('/\]\s*[\w ]*\w\W+(.+)/', $logfile_contents[$i], $matches) ) {
        $log[$n]->hash = hash( 'crc32' , $log[$n]->message );

        if ( ! in_array($log[$n]->hash, $log_hashes) ) {
          $log_hashes[] = $log[$n]->hash;
        }
      }
      else {
        $log[$n]->hash = '';
      }
    }
  }

  hybridsupply_log_count(array(
    'count' => count($log_hashes),
    'count_all' => count($log),
  ));
}

/*
 * Function: Log 404 Error
 */
function hybridsupply_log_404_errors(){
  if( is_404() ){
    $HTTP_REFERER = isset($_SERVER["HTTP_REFERER"]) ? $_SERVER["HTTP_REFERER"] : NULL;
    $url = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

    if ( $HTTP_REFERER && strpos( strtolower($url), 'cache') === FALSE ) {
      error_log('HTTP error (404): <a target="_blank" href="' . $url . '">' . $url . '</a>' . ' was not found. Request originated from <a target="_blank" href="' . $HTTP_REFERER . '">' . $HTTP_REFERER . '</a>.');
    }
  }
}

add_action( 'template_redirect', 'hybridsupply_log_404_errors' );
?>